import svgwrite

class FretBoardGtr():


    def __init__(self):
        self.tuning=['E','A','D','G','B','E'] # Default tuning
        self.fingering=[0,3,2,0,1,0]
        self._ol=10 # overlay
        self.square_size=500 # not important here because redefined

        self.gap=3 # minimum distance between max fret and min fret
        self.path="default.svg"

        self.dic_color={
            "1":'rgb(231, 0, 0)',
            "b2":'rgb(249, 229, 0)',
            "2":'rgb(249, 165, 0)',
            "b3":'rgb(0, 94, 0)',
            "3":'rgb(0, 108, 0)',
            "4":'rgb(0, 154, 0)',
            "b5":'rgb(0, 15, 65)',
            "5":'rgb(0, 73, 151)',
            "b6":'rgb(168, 107, 98)',
            "6":'rgb(222, 81, 108)',
            "b7":'rgb(120, 37, 134)',
            "7":'rgb(120, 25, 98)'
        }

        self.attribute_dic={
        'wf':50, #width of fret
        'hf':70, # height of fret
        'R':20, # radius of circle
        'background_color':'rgb(255,255,255)',
        'fretcolor':'rgb(150,150,150)',
        'strings_color':'rgb(0,0,0)',
        'nut_color':'rgb(0,0,0)',
        'fretsize':3,
        'string_same_size':False,
        'string_size':3,
        'dot_color':'rgb(200,200,200)',
        'dot_color_stroke':'rgb(0,0,0)',
        'dot_width_stroke':2,
        'dot_radius':7,
        'fontsize_bottom_tuning':15,
        'fontsize_text':20,
        'fontsize_fret':20,
        'open_circle_color':'rgb(255,255,255)',
        'open_circle_stroke_color':'rgb(0,0,0)',
        'open_circle_stroke_width':3,
        'open_text_color':'rgb(0,0,0)',
        'cross_color':'rgb(0,0,0)',
        'fretted_circle_color':'rgb(0,0,0)',
        'fretted_circle_stroke_color':'rgb(0,0,0)',
        'fretted_circle_stroke_width':3,
        'fretted_text_color':'rgb(255,255,255)',
        'fontsize_cross':20,
        'nut_height':7,
        'show_nut':True,
        'first_fret':0,
        'last_fret':12,
        'show_tun':True,
        'show_ft':True,
        'color_scale':True,
        'open_color_scale':False,
        'show_note_name':False,
        'show_degree_name':True,
        'color_chord':True,
        'open_color_chord':True
        }

        for i in self.attribute_dic.keys():
            setattr(self,i, self.attribute_dic[i])

    def pathname(self,path):
        """
        Take a parameter to set the path to save.
        >>> FretBoardGtr.pathname("test.svg")
        """
        self.path=path

    def layout(self):
        square_x=self.hf*(len(self.tuning)+2)
        pass

    def customtuning(self,tuning):
        '''
        Create a custom tuning

        >>> FretBoardGtr.customtuning(['D','A','D','G','A','D'])
        '''
        self.tuning=tuning

    def set_color(self,default_theme=False,**kwargs):
        '''
        Example :
        >>> FretboardGtr.set_color(root=rgb(0, 0, 0)')
        >>> FretboardGtr.set_color(default_theme=True)

        List of parameters : root, minorsecond, majorsecond, minorthird, majorthird,
                             perfectfourth, diminishedfifth, perfectfifth, minorsixth,
                             majorsixth,minorseventh,majorseventh
        '''

        dic_name={
        'root':"1",
        'minorsecond':'b2',
        'majorsecond':'2',
        'minorthird':'b3',
        'majorthird':"3",
        'perfectfourth':"4",
        'diminishedfifth':"b5",
        'perfectfifth':'5',
        'minorsixth':'b6',
        'majorsixth':"6",
        'minorseventh':"b7",
        'majorseventh':'7'
        }
        if default_theme:
            self.dic_color={
                "1":'rgb(231, 0, 0)',
                "b2":'rgb(249, 229, 0)',
                "2":'rgb(249, 165, 0)',
                "b3":'rgb(0, 94, 0)',
                "3":'rgb(0, 108, 0)',
                "4":'rgb(0, 154, 0)',
                "b5":'rgb(0, 15, 65)',
                "5":'rgb(0, 73, 151)',
                "b6":'rgb(168, 107, 98)',
                "6":'rgb(222, 81, 108)',
                "b7":'rgb(120, 37, 134)',
                "7":'rgb(120, 25, 98)'
            }
        else:

            for keys in kwargs:
                    if keys in dic_name.keys():
                        self.dic_color[dic_name[keys]]=kwargs[keys]
                    else:
                        print(keys,'is not a valid attribute.')

    def theme(self,default_theme=False,**kwargs):
        '''
        Set new attributes to the fretboard
                wf = 50 (width of fret)
                hf = 70 (height of fret)
                R = 20  (radius of circle )
                background_color = 'rgb(255,255,255)'
                fretcolor = 'rgb(150,150,150)'
                strings_color = 'rgb(0,0,0)'
                nut_color = 'rgb(0,0,0)'
                fretsize = 3
                string_same_size = False (If False first string bigger than the last)
                string_size = 3
                dot_color = 'rgb(200,200,200)'
                dot_color_stroke = 'rgb(0,0,0)' (small dots on the neck)
                dot_width_stroke = 2 (small dots on the neck)
                dot_radius = 7 (small dots on the neck)
                fontsize_bottom_tuning = 15 (fontsize of the tuning)
                fontsize_text = 20 (Fontsize of text in circle)
                open_circle_color = 'rgb(255,255,255)' (for open string)
                open_circle_stroke_color = 'rgb(0,0,0)' (for open string)
                open_circle_stroke_width = 3 (for open string)
                open_text_color = 'rgb(0,0,0)' (for open string)
                cross_color = 'rgb(0,0,0)' (Cross when doesn't play string )
                fontsize_cross = 20 (Cross when doesn't play string )
                fretted_circle_color = 'rgb(0,0,0)' (for fretted string)
                fretted_circle_stroke_color = 'rgb(0,0,0)' (for fretted string)
                fretted_circle_stroke_width = 3 (for fretted string)
                fretted_text_color = 'rgb(255,255,255)' (for fretted string)
                nut_height = 7
                show_nut = True
                first_fret = 0
                last_fret = 12
                show_tun = True
                show_ft = True
                color_scale = True (for the Scale class)
                show_note_name = False (if show_not_name==True, show_degree_name is not considered )
                show_degree_name = True
                color_chord=True (for the ChordGtr class)
                open_color_chord=True (for the ChordGtr class)
            '''
        if default_theme:
            for i in self.attribute_dic.keys():
                setattr(self,i, self.attribute_dic[i])
        else:
            for keys in kwargs:
                if keys in self.attribute_dic.keys():
                    setattr(self,keys,kwargs[keys])
                else:
                    print(keys,'is not a valid attribute.')


    def notesname(self):
        '''
        Give name of note on fret depends on tunings
        >>>FretBoardGtr.notesname(fingering=[0,3,2,0,None,0])
        ['E','C','E','G',None,'E']
        '''

        notes=[0]*len(self.tuning)
        chroma=["A","A#","B","C","C#","D","D#","E","F","F#","G","G#"]
        alter={
            "Bb":"A#",
            "Cb":"B",
            "B#":"C",
            "Db":"C#",
            "Eb":"D#",
            "Fb":"E",
            "Gb":"F#",
            "Ab":"G#"
        }

        for i in range(0,len(self.tuning)):
            if self.fingering[i] == None:
                notes[i]=None
            else:
                ind=chroma.index(self.tuning[i])
                notes[i]=chroma[(ind+self.fingering[i])%12]

        return notes


    def minmax(self):
        '''
        Return the min and the max of fingering without None and 0.
        FretBoardGtr.minmax(fingering=[0,3,2,0,None,0])
        (2,3)
        '''
        fretfing=[0 if v == None else v for v in self.fingering]
        return min(v for v in fretfing if v > 0),max(fretfing)

    def dist(self):
        '''
        return the gap between the max fret and the min fret
        >>> FretBoardGtr.dist(fingering=[0,3,2,0,None,0])
        1
        '''
        mini,maxi = self.minmax()
        self.gap=maxi-mini
        if self.gap <=3:
            self.gap=3
        return self.gap

    @staticmethod
    def find_intervals(scale=['C','E','G'],root='C'):
        '''
        >>> find_intervals(scale=['C','E','G'],root='C')
        [1,3,5]
        '''
        chroma=["A","A#","B","C","C#","D","D#","E","F","F#","G","G#"]
        intervals=["1","b2","2","b3","3","4","b5","5","b6","6","b7","7"]
        intervals_dic={
                "1":"root",
                "b2":"minor second",
                "2":"major second",
                "b3":"minor third",
                "3":"major third",
                "4":"perfect fourth",
                "b5":"diminished fifth",
                "5":"perfect fifth",
                "b6":"minor sixth",
                "6":"major sixth",
                "b7":"minor seventh",
                "7":"major seventh"

            }
        notes_string=[0]*len(chroma)
        for j in range(len(chroma)):
            #start from the root and place the other notes
            notes_string[j]=chroma[(chroma.index(root)+j)%12]
        inter=[0]*len(scale)
        for i in range(len(scale)):
            if scale[i]==None:
                inter[i]==None
            else:
                inter[i]=intervals[notes_string.index(scale[i])]
        return inter



    def wheredot(self,minfret):

        ''' Find where the dot on the fretboard are depends on the minimum fret
        If the first finger is on the 2rd fret (for example a B) and there is nothing beside the function return the folowwings lists:
        dot=[1,3]
        nbdot=[1,1]
        That leads to put a dot on the second fret and the fourth fret corresponding to the third and a fifth on the guitar.
        '''

        dot_fret=[0,3,5,7,9,12]
        nbdot_fret=[2,1,1,1,1,2]

        dot=[]
        nbdot=[]

        if self.gap>=12:

            for i in range(1,(self.gap//12)*len(dot_fret)):
                dot_fret.append(dot_fret[i]+12)
                nbdot_fret.append(nbdot_fret[i])

        for i in range(len(dot_fret)):
                    if dot_fret[i]>=(minfret)%12 and dot_fret[i]<minfret%12+self.gap+1:
                        dot.append(dot_fret[i]-minfret%12)
                        nbdot.append(nbdot_fret[i])

        if minfret==0:
            dot.pop(0)
            nbdot.pop(0)

        return dot, nbdot

    def save(self):
        self.dwg.save()



from fretboardgtr.fretboardgtr import FretBoardGtr #First name : main folder, second name .py file and last : class name
import svgwrite

class ChordGtr(FretBoardGtr):

    def __init__(self,fingering=[0,3,2,0,1,0],root='C'):
        FretBoardGtr.__init__(self)
        self.fingering=fingering
        self.root=root


    def fingering(self,fingering):
        self.fingering=fingering

    def wheredot(self,minfret):

        ''' Find where the dot on the fretboard are depends on the minimum fret
        If the first finger is on the 2rd fret (for example a B) and there is nothing beside the function return the folowwings lists:
        dot=[1,3]
        nbdot=[1,1]
        That leads to put a dot on the second fret and the fourth fret corresponding to the third and a fifth on the guitar.
        '''

        dot_fret=[0,3,5,7,9,12]
        nbdot_fret=[2,1,1,1,1,2]

        dot=[]
        nbdot=[]

        if self.gap>=12:

            for i in range(1,(self.gap//12)*len(dot_fret)):
                dot_fret.append(dot_fret[i]+12)
                nbdot_fret.append(nbdot_fret[i])

        for i in range(len(dot_fret)):
                    if dot_fret[i]>=(minfret)%12 and dot_fret[i]<minfret%12+self.gap+1:
                        dot.append(dot_fret[i]-minfret%12)
                        nbdot.append(nbdot_fret[i])

        if minfret==0:
            dot.pop(0)
            nbdot.pop(0)

        return dot, nbdot


    def emptybox(self):

            self.dwg = svgwrite.Drawing(
            self.path,
            size=(self.wf*(len(self.tuning)+2),self.hf*6+self.hf*(self.gap-3)),
            profile='tiny'
        )

    def background_fill(self):
        self.dwg.add(
            self.dwg.rect(
                insert=(self.wf+ self._ol, self.hf +self._ol),
                size=((len(self.tuning)-1)*self.wf, (len(self.tuning)-2)*self.hf), #-2 evite case du bas du tuning
                rx=None, ry=None,
                fill=self.background_color
            )
        )

    def background_fill_image(self):
        self.dwg.add(
            self.dwg.image("wood1.jfif",
                x=self.wf+ self._ol,
                y=self.hf +self._ol,
                width=(len(self.tuning)-1)*self.wf,
                height=(len(self.tuning)-2)*self.hf, #-2 evite case du bas du tuning
                preserveAspectRatio="none",
                opacity=0.90
                )
                )

    def add_dot(self):

        fretfing=[0 if v == None else v for v in self.fingering]
        minfret = min(v for v in fretfing if v > 0)


        dot,nbdot = self.wheredot(minfret)
        if max(fretfing)>4:
            for i in range(len(dot)):
                if nbdot[i]==1:
                    self.dwg.add(self.dwg.circle(((len(self.tuning)/2+1/2)*self.wf +self._ol,(1.5+dot[i])*self.hf+self._ol),r=self.dot_radius,fill=self.dot_color,stroke=self.dot_color_stroke,stroke_width=self.dot_width_stroke))
                if nbdot[i]==2:
                    self.dwg.add(self.dwg.circle(((len(self.tuning)/2 - 1/2)*self.wf +self._ol,(1.5+dot[i])*self.hf+self._ol),r=self.dot_radius,fill=self.dot_color,stroke=self.dot_color_stroke,stroke_width=self.dot_width_stroke))
                    self.dwg.add(self.dwg.circle(((len(self.tuning)/2 +1.5)*self.wf +self._ol,(1.5+dot[i])*self.hf+self._ol),r=self.dot_radius,fill=self.dot_color,stroke=self.dot_color_stroke,stroke_width=self.dot_width_stroke))
        else:
                    self.dwg.add(self.dwg.circle(((len(self.tuning)/2+1/2)*self.wf +self._ol,(3.5)*self.hf+self._ol),r=self.dot_radius,fill=self.dot_color,stroke=self.dot_color_stroke,stroke_width=self.dot_width_stroke))





    def createfretboard(self):
        '''
        Create an empty set of rectangles based on tunings.
        '''
        fretfing=[0 if v == None else v for v in self.fingering]

        #Creation of fret
        if max(fretfing)>4:
            for i in range(self.gap+2):
                #self.gap +2 : two is for the beginning and the end of the fretboard
                self.dwg.add(
                    self.dwg.line(
                        start=(self.wf +self._ol, (self.hf)*(i+1)+self._ol),
                        end=((self.wf)*(len(self.tuning))+self._ol,(self.hf)*(1+i)+self._ol),
                        stroke=self.fretcolor,
                        stroke_width=self.fretsize
                    )
                )
        else:
            for i in range(self.gap+1):
                #self.gap +1 :  for  the end of the fretboard and (i+2) to avoid first fret when nut
                self.dwg.add(
                    self.dwg.line(
                        start=(self.wf +self._ol, (self.hf)*(i+2)+self._ol),
                        end=((self.wf)*(len(self.tuning))+self._ol,(self.hf)*(i+2)+self._ol),
                        stroke=self.fretcolor,
                        stroke_width=self.fretsize
                    )
                )

        #creation of strings
        if self.string_same_size==False:
            string_size_list=[((self.string_size)-i/4) for i in range(len(self.tuning))]

        elif self.string_same_size==True:
            string_size_list=[(self.string_size) for i in range(len(self.tuning))]

        for i in range(len(self.tuning)):

            self.dwg.add(
                self.dwg.line(
                    start=((self.wf)*(1+i)+self._ol, self.hf+self._ol-self.fretsize/2),
                    end=((self.wf)*(1+i)+self._ol,self.hf+self._ol + (self.gap+1)*self.hf +self.fretsize/2),
                    stroke=self.strings_color,
                    stroke_width=string_size_list[i]
                )
            )
    def nut(self):
        '''
        Create nut if condition in fillfretboard.

        '''
        if self.string_same_size==False:
            self.dwg.add(
                self.dwg.line(
                    start=(self.wf +self._ol-((self.string_size))/2,(self.hf)*(1)+self._ol),
                    end=((self.wf)*(len(self.tuning))+self._ol +((self.string_size)-len(self.tuning)/4)/2 ,(self.hf)*(1)+self._ol ),
                    stroke=self.nut_color,
                    stroke_width=self.nut_height
                )
            )
        else:
            self.dwg.add(
                self.dwg.line(
                    start=(self.wf +self._ol-((self.string_size))/2,(self.hf)*(1)+self._ol),
                    end=((self.wf)*(len(self.tuning))+self._ol +((self.string_size))/2 ,(self.hf)*(1)+self._ol ),
                    stroke=self.nut_color,
                    stroke_width=self.nut_height
                )
            )
    def show_fret(self):
        '''
        Show text under the frets for example 3ft if condition in fillfretboard
        '''
        dot=[3,5,7,9,12,15,17,19,21,24]
        dot=[dot[v] for v in range(len(dot)) if dot[v]<=self.last_fret and dot[v]>=self.first_fret]
        #self.first_fret=0
        #self.last_fret=12
        if self.show_ft:
            for j,v in enumerate(dot):
                Y=self.wf*(1+len(self.tuning))
                X=self.hf*(1/2+v-self.first_fret)+self._ol
                t=svgwrite.text.Text(str(v)+"ft", insert=(X,Y),font_size=self.fontsize_text,font_weight="bold",style="text-anchor:middle;dominant-baseline:central")
                self.dwg.add(t)

    def show_tuning(self,fretfing):
        '''
        Show  tuning at the end of the neck.
        '''

        Max_after_conv=max([0 if v == None else v for v in self.fingering])
        if max(fretfing)>4:
            for i in range(len(self.tuning)):
                X=self.wf*(1+i)+self._ol
                Y=self.hf*(Max_after_conv+1/2+2)+self._ol

                t=svgwrite.text.Text(self.tuning[i], insert=(X,Y),font_size=self.fontsize_bottom_tuning,font_weight="normal",style="text-anchor:middle;dominant-baseline:central")
                self.dwg.add(t)

        else:
            for i in range(len(self.tuning)):
                X=self.wf*(1+i)+self._ol
                Y=self.hf*(5+1/2)+self._ol

                t=svgwrite.text.Text(self.tuning[i], insert=(X,Y),font_size=self.fontsize_bottom_tuning,font_weight="normal",style="text-anchor:middle;dominant-baseline:central")
                self.dwg.add(t)



    def fillfretboard(self):
        self.dist() #modify self.gap
        chroma=["A","A#","B","C","C#","D","D#","E","F","F#","G","G#"]
        intervals=["1","b2","2","b3","3","4","b5","5","b6","6","b7","7"]



        fingname=self.notesname()
        inter=FretBoardGtr.find_intervals(fingname,self.root)

        fretfing=[0 if v == None else v for v in self.fingering]
        minfret = min(v for v in fretfing if v > 0)

        if max(fretfing)>4:

            X=self.wf*(1+len(self.tuning))+self._ol
            Y=self.hf*(3/2)+self._ol
            t=svgwrite.text.Text(str(minfret)+"ft", insert=(X,Y),font_size=self.fontsize_fret,font_weight="bold",style="text-anchor:middle;dominant-baseline:central")
            self.dwg.add(t)

            fingering=[v if v==None else v-minfret+1 if v!=0 else v for v in self.fingering ]


        else:
            self.nut()
            fingering=self.fingering

        for i in range(0,len(self.tuning),1):

            if fingering[i]== None:
                X=self.wf*(1+i)+self._ol
                Y=self.hf*(1/2)+self._ol

                t=svgwrite.text.Text('X', insert=(X,Y),font_size=self.fontsize_cross,font_weight="bold",fill=self.cross_color,style="text-anchor:middle;dominant-baseline:central")
                self.dwg.add(t)
                #dwg.add(dwg.image("cross.svg",x=(i+1-0.3)*self.wf +self._ol,y=self.hf*(1/4-0.2)+self._ol,width=2*self.R))

            else:
                X=self.wf*(1+i)+self._ol
                Y=self.hf*(fingering[i]+1/2)+self._ol

                if fingering[i]==0:
                    if self.open_color_chord:
                        color=self.dic_color[inter[i]]
                    else:
                        color=self.fretted_circle_color
                    if self.show_note_name:
                        name_text=fingname[i]
                    elif self.show_degree_name:
                        name_text=str(inter[i])
                    else:
                        name_text=""
                    self.dwg.add(self.dwg.circle((X,Y),r=self.R,fill=self.open_circle_color,stroke=color,stroke_width=self.open_circle_stroke_width))
                    t=svgwrite.text.Text(name_text, insert=(X,Y),font_size=self.fontsize_text,font_weight="bold",fill=self.open_text_color,style="text-anchor:middle;dominant-baseline:central")
                    self.dwg.add(t)
                else:
                    if self.color_chord:
                        color=self.dic_color[inter[i]]
                    else:
                        color=self.fretted_circle_color
                    if self.show_note_name:
                        name_text=fingname[i]
                    elif self.show_degree_name:
                        name_text=str(inter[i])
                    else:
                        name_text=""

                    self.dwg.add(self.dwg.circle((X,Y),r=self.R,fill=color,stroke=self.fretted_circle_stroke_color,stroke_width=self.fretted_circle_stroke_width))
                    t=svgwrite.text.Text(name_text, insert=(X,Y),font_size=self.fontsize_text,fill=self.fretted_text_color,font_weight="bold",style="text-anchor:middle;dominant-baseline:central")
                    self.dwg.add(t)

            if self.show_tun:
                self.show_tuning(fretfing)


    def draw(self):
        self.emptybox()
        self.background_fill()
        #self.background_fill_image()
        self.add_dot()
        self.createfretboard()
        self.fillfretboard()


        return self.dwg



from fretboardgtr.fretboardgtr import FretBoardGtr #First name : main folder, second name .py file and last : class name
import svgwrite

class ScaleGtr(FretBoardGtr):
    def __init__(self,scale=['C','E','G'],root='C'):
        FretBoardGtr.__init__(self)
        self.scale=scale
        self.root=root


    def emptybox(self):
        '''
        Create empty box and the object self.dwg
        '''

        self.dwg = svgwrite.Drawing(
        self.path,
        size=(self.hf*(self.last_fret-self.first_fret+2),self.wf*(len(self.tuning)+2)),
        profile='tiny'
        )

    def background_fill(self):
        '''
        Fill background with a color
        '''
        self.dwg.add(
            self.dwg.rect(
                insert=(self.wf+ self._ol, self.hf +self._ol),
                size=((len(self.tuning)-1)*self.wf, (len(self.tuning)-2)*self.hf), #-2 evite case du bas du tuning
                rx=None, ry=None,
                fill=self.background_color
            )
        )

    def background_fill_image(self):
        '''
        Fill background with an image
        '''
        self.dwg.add(
            self.dwg.image("wood1.jfif",
                x=self.wf+ self._ol,
                y=self.hf +self._ol,
                width=(len(self.tuning)-1)*self.wf,
                height=(len(self.tuning)-2)*self.hf, #-2 evite case du bas du tuning
                preserveAspectRatio="none",
                opacity=0.90
                )
                )

    def add_dot(self):
        '''
        Add dot up to 24 frets.
        Recalculate if the minimum fret isn't 0 or maximum fret isn't 12.
        '''

        dot=[3,5,7,9,12,15,17,19,21,24]
        dot=[dot[v] for v in range(len(dot)) if dot[v]<=self.last_fret and dot[v]>=self.first_fret]
        for i in range(len(dot)):
            if dot[i]%12==0:
                self.dwg.add(
                    self.dwg.circle(
                        ((0.5+dot[i]-self.first_fret)*self.hf+self._ol,(len(self.tuning)/2 - 1/2)*self.wf +self._ol),
                        r=self.dot_radius,
                        fill=self.dot_color,
                        stroke=self.dot_color_stroke,
                        stroke_width=self.dot_width_stroke
                    )
                )

                self.dwg.add(
                    self.dwg.circle(
                        ((0.5+dot[i]-self.first_fret)*self.hf+self._ol,(len(self.tuning)/2 + 3/2)*self.wf +self._ol),
                        r=self.dot_radius,
                        fill=self.dot_color,
                        stroke=self.dot_color_stroke,
                        stroke_width=self.dot_width_stroke
                    )
                )

            else:
                self.dwg.add(
                    self.dwg.circle(
                        ((0.5+dot[i]-self.first_fret)*self.hf+self._ol,(len(self.tuning)/2+1/2)*self.wf +self._ol),
                        r=self.dot_radius,
                        fill=self.dot_color,
                        stroke=self.dot_color_stroke,
                        stroke_width=self.dot_width_stroke))




    def createfretboard(self):
        '''
        Create an sets of lines based on tunings and the number of fret.
        '''
        string_size_list=[((self.string_size)-i/4) for i in range(len(self.tuning))]
        string_size_list=list(reversed(string_size_list))
        g=0

        if self.first_fret!=0:
            self.show_nut=False

        if self.show_nut:
            g=1
        #begin before if min fret !=0 because no open chords
        if self.first_fret!=0:
            it=0
            nb=2
        else:
            it=1
            nb=1
        for i in range(g,(self.last_fret-self.first_fret)+nb):
            self.dwg.add(
                self.dwg.line(
                    start=((self.hf)*(i+it)+self._ol,self.wf +self._ol),
                    end=((self.hf)*(i+it)+self._ol,(self.wf)*(len(self.tuning))+self._ol),
                    stroke=self.fretcolor,
                    stroke_width=self.fretsize
                )
            )

        #strings
        if self.string_same_size:
            string_size_list=[((self.string_size)) for i in range(len(self.tuning))]
        #begin before if min fret !=0 because no open chords
        if self.first_fret!=0:
            xstart=self._ol-self.fretsize/2
            xend=self._ol + ((self.last_fret-self.first_fret+1))*self.hf +self.fretsize/2
        else:
            xstart=self.hf+self._ol-self.fretsize/2
            xend=self.hf+self._ol + (self.last_fret-self.first_fret)*self.hf +self.fretsize/2

        for i in range(len(self.tuning)):

            self.dwg.add(
                self.dwg.line(
                    start=(xstart,(self.wf)*(1+i)+self._ol),
                    end=(xend,(self.wf)*(1+i)+self._ol),
                    stroke=self.strings_color,
                    stroke_width=string_size_list[i]
                )
            )

    def nut(self):
        '''
        Create nut if minimum fret == 0.

        '''
        if self.first_fret!=0:
            self.show_nut=False
        if self.show_nut:
            if self.string_same_size==False:
                self.dwg.add(
                    self.dwg.line(
                    start=((self.hf)*(1)+self._ol,self.wf +self._ol-((self.string_size)-len(self.tuning)/4)/2),
                    end=((self.hf)*(1)+self._ol,(self.wf)*(len(self.tuning))+self._ol +((self.string_size))/2 ),
                    stroke=self.nut_color,
                    stroke_width=self.nut_height
                )
            )
            else:
                self.dwg.add(
                    self.dwg.line(
                    start=((self.hf)*(1)+self._ol,self.wf +self._ol-((self.string_size))/2),
                    end=((self.hf)*(1)+self._ol,(self.wf)*(len(self.tuning))+self._ol +((self.string_size))/2 ),
                    stroke=self.nut_color,
                    stroke_width=self.nut_height
                )
            )
    def show_fret(self):
        '''
        Show text under the frets for example 3ft.
        '''
        dot=[3,5,7,9,12,15,17,19,21,24]
        dot=[dot[v] for v in range(len(dot)) if dot[v]<=self.last_fret and dot[v]>=self.first_fret]
        #self.first_fret=0
        #self.last_fret=12
        if self.show_ft:
            for j,v in enumerate(dot):
                Y=self.wf*(1+len(self.tuning))
                X=self.hf*(1/2+v-self.first_fret)+self._ol
                t=svgwrite.text.Text(str(v)+"ft", insert=(X,Y),font_size=self.fontsize_fret,font_weight="bold",style="text-anchor:middle;dominant-baseline:central")
                self.dwg.add(t)

    def show_tuning(self):
        '''
        Show  tuning at the end of the neck.
        '''

        if self.show_tun:
            reverse_tun=list(reversed(self.tuning))
            for i in range(len(self.tuning)):
                Y=self.wf*(1+i)+self._ol
                X=self.hf*(self.last_fret-self.first_fret+3/2)+self._ol

                t=svgwrite.text.Text(reverse_tun[i], insert=(X,Y),font_size=self.fontsize_bottom_tuning,font_weight="normal",style="text-anchor:middle;dominant-baseline:central")
                self.dwg.add(t)


    def fill_with_chords(self):

        chroma=["A","A#","B","C","C#","D","D#","E","F","F#","G","G#"]
        intervals=["1","b2","2","b3","3","4","b5","5","b6","6","b7","7"]



        self.dist()

        fingname=self.notesname()
        fingname=list(reversed(fingname))
        fretfing=[0 if v == None else v for v in self.fingering]
        minfret = min(v for v in fretfing if v > 0)

        if max(fretfing)>12:
            return None

        fingering=[v if v==None else v-minfret+1 if v!=0 else v for v in self.fingering ]
        fingering=list(reversed(fingering))

        for i in range(0,len(self.tuning),1):

            if fingering[i]== None:
                Y=self.wf*(1+i)+self._ol
                X=self.hf*(1/2)+self._ol

                t=svgwrite.text.Text('X', insert=(X,Y),font_size=self.fontsize_cross,font_weight="bold",fill=self.cross_color,style="text-anchor:middle;dominant-baseline:central")
                self.dwg.add(t)
                #dwg.add(dwg.image("cross.svg",x=(i+1-0.3)*self.wf +self._ol,y=self.hf*(1/4-0.2)+self._ol,width=2*self.R))

            else:

                Y=self.wf*(1+i)+self._ol
                X=self.hf*(fingering[i]+1/2)+self._ol

                if fingering[i]==0:

                    self.dwg.add(self.dwg.circle((X,Y),r=self.R,fill=self.open_circle_color,stroke=self.open_circle_stroke_color,stroke_width=self.open_circle_stroke_width))
                    t=svgwrite.text.Text(fingname[i], insert=(X,Y),font_size=self.fontsize_text,font_weight="bold",fill=self.open_text_color,style="text-anchor:middle;dominant-baseline:central")
                    self.dwg.add(t)

                else:

                    self.dwg.add(self.dwg.circle((X,Y),r=self.R,fill=self.fretted_circle_color,stroke=self.fretted_circle_stroke_color,stroke_width=self.fretted_circle_stroke_width))
                    t=svgwrite.text.Text(fingname[i], insert=(X,Y),font_size=self.fontsize_text,fill=self.fretted_text_color,font_weight="bold",style="text-anchor:middle;dominant-baseline:central")
                    self.dwg.add(t)




        return self.dwg


    def fill_with_scale(self):
        '''
        Fill the neck with scale like ["C","D","E","F","G","A","B"]
        automatically calculate position and name depends on tunings and number of fret.
        '''
        chroma=["A","A#","B","C","C#","D","D#","E","F","F#","G","G#"]
        intervals=["1","b2","2","b3","3","4","b5","5","b6","6","b7","7"]
        notes_from_root=[0]*len(chroma)
        if self.first_fret!=0:
            nb=1
        else:
            nb=1
        for j in range(len(chroma)):
            #start from the root and place the other notes
            notes_from_root[j]=chroma[(chroma.index(self.root)+j)%12]

        color=self.fretted_circle_color
        color_stroke=self.open_circle_stroke_color


        for l in range(3):
            m=0

            for i in reversed(range(len(self.tuning))):

                nt=self.tuning[i]
                notes_string=[0]*12

                for j in range(12):#13 beacause add the 12th fret

                    if (12*l+j)>(self.last_fret):
                        break

                    notes_string[j]=chroma[(chroma.index(nt)+j)%12]

                for k in range(len(self.scale)):

                    try:
                        index=notes_string.index(self.scale[k])
                    except:
                        continue  #Continue return to the beginning of the loop without executing the code after (contrarly to pass)

                    Y=self.wf*(1+m)+self._ol
                    X=self.hf*(12*l+index+1/2-self.first_fret)+self._ol

                    if self.show_note_name:

                        if (index+l)==0:
                            if self.open_color_scale:
                                inter=FretBoardGtr.find_intervals(self.scale,self.root)
                                color_stroke=self.dic_color[inter[k]]

                            self.dwg.add(self.dwg.circle((X,Y),r=self.R,fill=self.open_circle_color,stroke=color_stroke,stroke_width=self.open_circle_stroke_width))
                            t=svgwrite.text.Text(notes_string[index], insert=(X,Y),font_size=self.fontsize_text,font_weight="bold",fill=self.open_text_color,style="text-anchor:middle;dominant-baseline:central")
                            self.dwg.add(t)

                        else:
                            if self.color_scale:
                                inter=FretBoardGtr.find_intervals(self.scale,self.root)
                                color=self.dic_color[inter[k]]

                            self.dwg.add(self.dwg.circle((X,Y),r=self.R,fill=color,stroke=self.fretted_circle_stroke_color,stroke_width=self.fretted_circle_stroke_width))
                            t=svgwrite.text.Text(notes_string[index], insert=(X,Y),font_size=self.fontsize_text,font_weight="bold",fill=self.fretted_text_color,style="text-anchor:middle;dominant-baseline:central")
                            self.dwg.add(t)

                    elif self.show_degree_name:

                        if (index+l)==0:
                            if self.open_color_scale:
                                inter=FretBoardGtr.find_intervals(self.scale,self.root)
                                color_stroke=self.dic_color[inter[k]]
                            self.dwg.add(self.dwg.circle((X,Y),r=self.R,fill=self.open_circle_color,stroke=color_stroke,stroke_width=self.open_circle_stroke_width))
                            t=svgwrite.text.Text(intervals[notes_from_root.index(notes_string[index])], insert=(X,Y),font_size=self.fontsize_text,font_weight="bold",fill=self.open_text_color,style="text-anchor:middle;dominant-baseline:central")
                            self.dwg.add(t)

                        else:
                            if self.color_scale:
                                inter=FretBoardGtr.find_intervals(self.scale,self.root)
                                color=self.dic_color[inter[k]]

                            self.dwg.add(self.dwg.circle((X,Y),r=self.R,fill=color,stroke=self.fretted_circle_stroke_color,stroke_width=self.fretted_circle_stroke_width))
                            t=svgwrite.text.Text(intervals[notes_from_root.index(notes_string[index])], insert=(X,Y),font_size=self.fontsize_text,font_weight="bold",fill=self.fretted_text_color,style="text-anchor:middle;dominant-baseline:central")
                            self.dwg.add(t)
                    else:

                        if (index+l)==0:
                            if self.open_color_scale:
                                inter=FretBoardGtr.find_intervals(self.scale,self.root)
                                color_stroke=self.dic_color[inter[k]]

                            self.dwg.add(self.dwg.circle((X,Y),r=self.R,fill=self.open_circle_color,stroke=color_stroke,stroke_width=self.open_circle_stroke_width))

                        else:
                            if self.color_scale:
                                inter=FretBoardGtr.find_intervals(self.scale,self.root)
                                color=self.dic_color[inter[k]]

                            self.dwg.add(self.dwg.circle((X,Y),r=self.R,fill=color,stroke=self.fretted_circle_stroke_color,stroke_width=self.fretted_circle_stroke_width))

                m+=1

        return self.dwg



    def draw(self,fingering=[]):
        """
        Execute in particular order each important methods to create the board
        """
        self.emptybox()
        self.nut()
        #self.background_fill()
        #self.background_fill_image()
        self.add_dot()
        self.createfretboard()
        self.show_fret()
        self.show_tuning()
        if len(fingering)==0:
            self.fill_with_scale()
        else:
            self.fingering=fingering
            self.fill_with_chords()

if __name__=='__main__':
    F=ScaleGtr()
    F.theme(default_theme=True)
    F.customtuning(['F','A','D','G','B','E'])
    F.set_color(root='red')
    F.draw()
    F.save()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="fretboardgtr", # Replace with your own username
    version="0.0.1",
    author="Ant Gib",
    author_email="ant.gib@protonmail.com",
    description="This is a package for creating chords and fretboard visualisation of scale in svg format.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/antscloud/fretboardgtr/",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    install_requires=[
          'svgwrite',
      ],
    python_requires='>=3',
)
